DM "CLEAR OUTPUT;CLEAR LOG";
*__________________________________________________________________________________________________
Project: HCBS Quality Measures built under the Money Follows the Person (MFP) Contract
Program: 01_Prep_Denominator.sas

Purpose: This program is to prepare the denominator file, keep key variables

Authors:  HXU and XLIN at Mathematica Policy Research 10/6/2014
Input:    Analytic file with person-level information on HCBS use and enrollment, demographics, and risk factors
Output:  Denominator file restricted to only those variables needed

Notes:    1) The input analytic file should have all the variables needed, including risk factors that are re-merged in 04_RISK_ADJUST.SAS
___________________________________________________________________________________________________;
OPTIONS NOCENTER FORMDLIM='=' LS=256 ORIENTATION=LANDSCAPE OBS=MAX COMPRESS=YES;

FILENAME CONTROL 'CONTROL.SAS'; *<===USER MUST modify;
%INCLUDE CONTROL;

LIBNAME IN  "&PATH.";*user modify;
LIBNAME OUT "&PATH.";*user modify;

%LET INFILE  =IN.HCBS_analytic_file;     *user modify; 
%LET OUTFILE =OUT.HCBS_PQI_DENOM;        *user modify; 

PROC FORMAT;
   *AGE format;
   VALUE AGEFMT 18-24 ='18-24'
                25-44 ='25-44'
                45-64 ='45-64'
                65-74 ='65-74'
                75-84 ='75-84'
                85-HIGH='>=85';

RUN;

TITLE1 "01_PREP_DENOMINATOR.SAS";

******************************************************************************************************************
Read in file
******************************************************************************************************************;
DATA MERGED_DENOM;
   SET &INFILE.;

   *Create the Age range variables;
   AGE_18_24 = 0;
   AGE_25_44 = 0;
   AGE_45_64 = 0;
   AGE_65_74 = 0;
   AGE_75_84 = 0;
   AGE_85    = 0;

   IF 18 <= A_AGE <= 24 THEN AGE_18_24 = 1;
   IF 25 <= A_AGE <= 44 THEN AGE_25_44 = 1;
   IF 45 <= A_AGE <= 64 THEN AGE_45_64 = 1;  
   IF 65 <= A_AGE <= 74 THEN AGE_65_74 = 1;
   IF 75 <= A_AGE <= 84 THEN AGE_75_84 = 1;
   IF  A_AGE >= 85      THEN AGE_85    = 1;
RUN;

TITLE2 "Check A_AGE";
PROC FREQ DATA=MERGED_DENOM;
   TABLES A_AGE*AGE_18_24*AGE_25_44*AGE_45_64*AGE_65_74*AGE_75_84*AGE_85/MISSING LIST;
   FORMAT A_AGE AGEFMT.;
RUN;

*OUTFILE: Restrict denominator file to only needed variables;
DATA &OUTFILE.(KEEP=MSIS_ID MAX_STATE HICNO CALENDAR_YR
                     A_FEMALE A_DUAL A_AGE A_RACE 
                     AGE_18_24 AGE_25_44 AGE_45_64 AGE_65_74 AGE_75_84 AGE_85 
                     HCBS_TIME HCBS_TIME_PRU
                     HCBS_TIME_FLAG_1 - HCBS_TIME_FLAG_12                    
                     HCBS_USER_1-HCBS_USER_12
                     MAX_HCBS_ENROLL_1-MAX_HCBS_ENROLL_12
                     EL_MDCR_DUAL_MO_1 -EL_MDCR_DUAL_MO_12
                     HOSPICE_START HOSPICE_END HCBS_TIME_PRU 

                     ACP ANXI BIPL DEPSN PSDS PTRA SCHI SCHIOT TOBA ASD CP CF EPLPSY ID LD MOBIMP MS MDYS
                     ODD DEAF BLIND SPINA SCI TBI ALCO SUBS HYPTHY AMI ALZ ALZRDSD ANEMIA ASTHMA AFIB
                     HYPPLA CAT CKD COPD DEPR DIAB GLCM CHF HFRAC HYPLIP HYPTEN IHD OST RA STRK
                     BRC CRC PRC LNGC ENDC
                 );
   SET MERGED_DENOM;
RUN;

TITLE2 "Contents of &OUTFILE.";
PROC CONTENTS DATA=&OUTFILE.;RUN;

/*The End*/
