DM "CLEAR OUTPUT;CLEAR LOG";
*__________________________________________________________________________________________________
Project: Money Follows Person(MFP)
Program: 04_RISK_ADJUST.sas
Purpose: STATE LEVEL OBSERVED, EXPECTED, RISK ADJUSTED AND OVEALL AVERAGE FROM 2010 coeffients FOR PQI91-PQI93
Input:   a person-level file with risk factors and outcomes(TAPQ) variables already defined    
Output:  ACUTE_MME
         ACUTE_MED_ONLY
         CHONRIC_MME
         CHONRIC_MED_ONLY
         PRU_MME
         PRU_MED_ONLY

___________________________________________________________________________________________________;
OPTIONS NOCENTER FORMDLIM='=' LS=256 ORIENTATION=LANDSCAPE OBS=MAX;
FILENAME CONTROL 'CONTROL.SAS'; *<===USER MUST modify;
%INCLUDE CONTROL;

*input dataset is a person-level file with risk factors and outcomes(TAPQ) variables already defined;
libname  IN       "&PATH.";
libname  INPARAM  "&PATH.";*2010 coefficient;
libname  OUT "&PATH.";

TITLE "RISK_ADJUST";

%LET INPT=HCBS_PQI_Person_Level;     *name can be changed;

**************************************************************************************************
*  LIST OF COVARIATES                                                                       *
*************************************************************************************************;
%LET DEMO      =    A_FEMALE AGE_18_24   AGE_25_44  AGE_45_64   AGE_75_84   AGE_85 ;

*COMP91_DUALS;
%LET COUNT_COVAR_91_DUAL=
  ACP    ALCO  ALZRDSD  ANXI    ASTHMA   ASD    BIPL     BLIND  BRC    CF    CHF    CKD    COPD   CP    CRC   /*DEAF*/ 
  DEPSN  DIAB  ENDC     EPLPSY  HFRAC   HYPPLA  HYPTEN   ID     LD     /*LNGC*/  MOBIMP MS     /*ODD*/    PSDS  PTRA 
  SCHIOT SCI   SPINA    STRK    SUBS    /*TBI*/     TOBA;

*COMP91_NON_DUALS;
%LET COUNT_COVAR_91_NON_DUAL=
  /*ACP*/    ALCO  ALZRDSD  ANXI    ASTHMA   /*ASD*/    BIPL     /*BLIND*/  BRC    CF    CHF    CKD    COPD   CP    /*CRC*/   DEAF 
  DEPSN  DIAB  /*ENDC*/     EPLPSY  HFRAC   HYPPLA  HYPTEN   ID     /*LD*/     /*LNGC*/  MOBIMP MS     ODD    /*PSDS*/  PTRA 
  SCHIOT SCI   SPINA    STRK    SUBS    TBI     TOBA;

*COMP92_DUALS;
%LET COUNT_COVAR_92_DUAL=
ACP     AFIB   ALCO     ALZRDSD   AMI     ANXI     ASD     ASTHMA  BIPL   BLIND    CF      CHF       
CKD     COPD   CP       DEAF      DEPSN   DIAB     EPLPSY  HFRAC  
HYPTEN  ID     IHD      LD        MDYS    MOBIMP   MS      ODD       
PSDS   PTRA    SCHIOT   SCI       SPINA   STRK     SUBS    TBI    TOBA  ; 

*COMP92_NON_DUALS;
%LET COUNT_COVAR_92_NON_DUAL=
ACP     AFIB   /*ALCO*/     ALZRDSD   AMI     ANXI    /* ASD*/     ASTHMA  BIPL   /*BLIND*/    CF      CHF       
CKD     COPD   CP       DEAF      DEPSN   DIAB     EPLPSY  HFRAC  
HYPTEN  ID     IHD      LD        MDYS    MOBIMP   MS      /*ODD*/       
PSDS   PTRA    SCHIOT   SCI       SPINA   STRK     SUBS    /*TBI*/    TOBA  ;

*COMP93_DUALS;
%LET COUNT_COVAR_93_DUAL=
  ACP    ALCO  ALZRDSD  ANXI    ASTHMA  ASD    BIPL     BLIND   /*BRC*/   CF    CHF    CKD    COPD   CP    /*CRC*/    DEAF 
  DEPSN  DIAB  ENDC     EPLPSY  HFRAC   HYPPLA  HYPTEN   ID     LD     /*LNGC*/  MOBIMP MS    ODD    PSDS  PTRA 
  SCHIOT SCI   SPINA    STRK    SUBS    TBI     TOBA;

*COMP93_NON_DUALS;
%LET COUNT_COVAR_93_NON_DUAL=
  ACP    ALCO  ALZRDSD  ANXI    /*ASTHMA*/  ASD    BIPL     BLIND   /*BRC*/  CF    CHF    CKD    COPD   /*CP*/    CRC     DEAF 
  DEPSN  DIAB  ENDC     EPLPSY  HFRAC   HYPPLA  HYPTEN   ID   /*LD     LNGC*/  MOBIMP MS    ODD    PSDS  /*PTRA*/
  SCHIOT SCI   SPINA    STRK    SUBS    TBI     TOBA;


DATA &INPT._0 &INPT._1;
  SET IN.&INPT.;
  *NEW;
  IF  AGE_65_74= 1 THEN  AGE_65_74=0;/* AGE GROUP 65_74 IS TREATED AS REFRENCE GROUP*/ 
  IF A_FEMALE ^= .;   /*EXCLUDE OBS MISSING GENDER INFO.*/
  IF A_AGE>=18;

  IF A_DUAL=0 THEN OUTPUT &INPT._0;
  ELSE IF A_DUAL=1 THEN OUTPUT &INPT._1;
RUN;

%MACRO TAPQ9192_ZINB(COMP_NUM=,COUNT_VAR=,MSR=,SUFFIX=);
DATA _&COMP_NUM._A_DUAL_&SUFFIX. (keep=STATE ID2 A_AGE A_DUAL  AGE_65_74  &DEMO. &COUNT_VAR.  &MSR. HCBS_TIME)  ;
  set &INPT._&SUFFIX.;
  ID2=_N_;
RUN;

DATA _NULL_;
   SET INPARAM.Params_&COMP_NUM._A_DUAL_&SUFFIX.(WHERE =(PARAMETER="Dispersion"));
   KEEP ESTIMATE;
   CALL SYMPUT ('K',ESTIMATE);
RUN;

*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
part 1: score non zero count
*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^;
PROC TRANSPOSE DATA=INPARAM.Params_&COMP_NUM._A_DUAL_&SUFFIX.  OUT=Params_&COMP_NUM._A_DUAL_&SUFFIX.  ;
VAR estimate ;
ID parameter;
RUN ;

data Params_&COMP_NUM._A_DUAL_&SUFFIX.; 
  set Params_&COMP_NUM._A_DUAL_&SUFFIX.;
  _NAME_ = "MHAT" ;
  _TYPE_ = "PARMS" ;
run;

*-- APPLY PROC SCORE TO DATA --* ;                                         
 PROC   SCORE DATA=_&COMP_NUM._A_DUAL_&SUFFIX. SCORE=Params_&COMP_NUM._A_DUAL_&SUFFIX. TYPE=PARMS OUT=scoreD_&COMP_NUM._A_DUAL_&SUFFIX.;
 VAR    &DEMO. &COUNT_VAR. ;
 RUN;

 *-- CALCULATE PREDICTED VALUES (EHAT) --* ;
 DATA   scoreD_&COMP_NUM._A_DUAL_&SUFFIX. ; 
 SET    scoreD_&COMP_NUM._A_DUAL_&SUFFIX.;
 EHAT = EXP(MHAT);
 RUN;

*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
part 2: score zero count
*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^;
PROC TRANSPOSE DATA=INPARAM.zero_paramest_&COMP_NUM._A_DUAL_&SUFFIX.  OUT=zero_paramest_&COMP_NUM._A_DUAL_&SUFFIX. ;
VAR estimate ;
ID parameter;
RUN ;

data zero_paramest_&COMP_NUM._A_DUAL_&SUFFIX.; 
  set zero_paramest_&COMP_NUM._A_DUAL_&SUFFIX.;
  _NAME_ = "MHAT" ;
  _TYPE_ = "PARMS" ;
run;


*-- APPLY PROC SCORE TO DATA --* ;                                          
 PROC   SCORE DATA=_&COMP_NUM._A_DUAL_&SUFFIX. SCORE=zero_paramest_&COMP_NUM._A_DUAL_&SUFFIX. TYPE=PARMS OUT=zero_scoreD_&COMP_NUM._A_DUAL_&SUFFIX.;
 VAR    &DEMO. &COUNT_VAR. ;
 RUN;

 *-- CALCULATE PREDICTED VALUES (EHAT) --* ;
 DATA   zero_scoreD_&COMP_NUM._A_DUAL_&SUFFIX. (keep=ID2 MHAT EHAT) ;
 SET    zero_scoreD_&COMP_NUM._A_DUAL_&SUFFIX.;

 EHAT = EXP(MHAT);
 RUN;

*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
part 3: merge zero count and non zero count scored data sets
*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^;
proc sort data =scoreD_&COMP_NUM._A_DUAL_&SUFFIX. ;
by ID2;
run;

proc sort data =zero_scoreD_&COMP_NUM._A_DUAL_&SUFFIX.  out= zero_scored_&COMP_NUM._A_DUAL_&SUFFIX.(rename=(EHAT=zero_EHAT MHAT=zero_MHAT));
by ID2;
run;

data scoreD_both_&COMP_NUM._A_DUAL_&SUFFIX.;
merge scoreD_&COMP_NUM._A_DUAL_&SUFFIX.(rename=(EHAT=NB_EHAT))  zero_scored_&COMP_NUM._A_DUAL_&SUFFIX.;
by ID2;
pzero    = (zero_EHAT)/(1+zero_EHAT);

PRED  = NB_EHAT*(1-pzero);


K=&K.;
**Calculate the variance for ZINB measure;
p_suc     = 1/(1+k*NB_EHAT);
r         = 1/k;
var_nb    = (1-p_suc)*r/(p_suc**2);
mean_zinb = (1-pzero)*NB_EHAT;
PHAT      = (1-pzero)*(NB_EHAT**2+var_nb)-(1-pzero)**2*NB_EHAT**2; /*END OF ADDTION*/;
run;

%MEND TAPQ9192_ZINB;

%TAPQ9192_ZINB(COMP_NUM=91,COUNT_VAR=&COUNT_COVAR_91_NON_DUAL.,  MSR=ACUTE,SUFFIX=0);
%TAPQ9192_ZINB(COMP_NUM=91,COUNT_VAR=&COUNT_COVAR_91_DUAL.,      MSR=ACUTE,SUFFIX=1);

%TAPQ9192_ZINB(COMP_NUM=92,COUNT_VAR=&COUNT_COVAR_92_NON_DUAL.,  MSR=CHRONIC,SUFFIX=0);
%TAPQ9192_ZINB(COMP_NUM=92,COUNT_VAR=&COUNT_COVAR_92_DUAL.,      MSR=CHRONIC,SUFFIX=1);


%MACRO TAPQ93_LOGISTIC(COMP_NUM=,COUNT_VAR=,MSR=,SUFFIX=);
DATA SCORED_&COMP_NUM._A_DUAL_&SUFFIX. (keep=STATE A_AGE A_DUAL AGE_65_74  &DEMO. &COUNT_VAR.  &MSR. HCBS_TIME_PRU)  ;
  set &INPT._&SUFFIX.;
run;

PROC TRANSPOSE DATA=INPARAM.Params_&COMP_NUM._A_DUAL_&SUFFIX.(KEEP=Variable Estimate) OUT=Params_&COMP_NUM._A_DUAL_&SUFFIX.(DROP=_NAME_);
   ID Variable;
   VAR Estimate;
RUN;
DATA Params_&COMP_NUM._A_DUAL_&SUFFIX.;
   SET Params_&COMP_NUM._A_DUAL_&SUFFIX.;
    _NAME_='MHAT' ;
    _TYPE_='PARMS' ;
RUN;

PROC SCORE DATA=SCORED_&COMP_NUM._A_DUAL_&SUFFIX. SCORE=Params_&COMP_NUM._A_DUAL_&SUFFIX. TYPE=PARMS OUT=out_&COMP_NUM._A_DUAL_&SUFFIX.;
   VAR &DEMO. &COUNT_VAR. ;
RUN;
DATA out_&COMP_NUM._A_DUAL_&SUFFIX.(DROP=MHAT);
  SET out_&COMP_NUM._A_DUAL_&SUFFIX.;
  PRED = EXP(MHAT)/(1 + EXP(MHAT));
  PHAT = PRED * (1-PRED);  
RUN;
%MEND TAPQ93_LOGISTIC;
%TAPQ93_LOGISTIC(COMP_NUM=93,COUNT_VAR=&COUNT_COVAR_93_NON_DUAL.,  MSR=PRU,SUFFIX=0);
%TAPQ93_LOGISTIC(COMP_NUM=93,COUNT_VAR=&COUNT_COVAR_93_DUAL.,      MSR=PRU,SUFFIX=1);


%MACRO STATELEVEL(PREFIX=,COMP_NUM=,MSR=,SUFFIX=,SUFFIX2=); 
TITLE2 "MSR=&MSR., Dual_Status=&SUFFIX." ;
DATA _&PREFIX._&COMP_NUM._A_DUAL_&SUFFIX.; SET &PREFIX._&COMP_NUM._A_DUAL_&SUFFIX.;
    EHAT=PRED;
    PWHT=1; 
    %IF  &COMP_NUM.=91 OR &COMP_NUM.=92 %THEN %DO;
    HCBS_TIME12=HCBS_TIME/12;   
    %END;
    %ELSE %IF &COMP_NUM.=93 %THEN %DO;
    HCBS_TIME_PRU12=HCBS_TIME_PRU/12;
    %END;
RUN;

PROC MEANS DATA=_&PREFIX._&COMP_NUM._A_DUAL_&SUFFIX. SUM MEAN VAR;
    CLASS STATE;
    VAR &MSR.  EHAT PWHT  
    %IF  &COMP_NUM.=91 OR &COMP_NUM.=92 %THEN %DO;
    HCBS_TIME12    
    %END;
    %ELSE %IF &COMP_NUM.=93 %THEN %DO;
    HCBS_TIME_PRU12 
    %END;

    PHAT 
    ;
    OUTPUT OUT=_&PREFIX._&COMP_NUM._A_DUAL_&SUFFIX._SUM SUM =&MSR.       EHAT      PWHT       DENOM        PHAT
                                                        MEAN=&MSR._MEAN  EHAT_MEAN PWHT_MEAN  DENOM_MEAN   PHAT_MEAN
                                                        VAR =&MSR._VAR   EHAT_VAR  PWHT_VAR   DENOM_VAR    PHAT_VAR
RUN;
DATA _&PREFIX._&COMP_NUM._A_DUAL_&SUFFIX._SUM (DROP=PWHT &MSR. YBAR_  O_E_); 
   RETAIN YBAR_  O_E_;
   SET _&PREFIX._&COMP_NUM._A_DUAL_&SUFFIX._SUM (KEEP =STATE &MSR. EHAT PWHT DENOM PHAT);
    
   NUM = ROUND(&MSR.,1);
   USERS= ROUND(PWHT,1);
   OR   = 100000*NUM / DENOM;
   ER   = 100000*EHAT/ DENOM;
   IF STATE= " " THEN DO ;
     YBAR_=OR;
     O_E_=(OR/ER);  
   END;
   IF STATE^= '' THEN DO;    
    RAR=(OR/ER)*YBAR_; 
   END;

   SD_RAR = 100000*sqrt((YBAR_/ER)**2*(PHAT/USERS**2));

   LRAR   = RAR - (1.96 * SD_RAR);
   IF LRAR < 0 THEN LRAR = 0;
   URAR   = RAR + (1.96 * SD_RAR); 

RUN;
DATA  OUT.&MSR._&SUFFIX2. ; 
    RETAIN STATE  NUM USERS DENOM OR  EHAT ER RAR  ; 
    SET _&PREFIX._&COMP_NUM._A_DUAL_&SUFFIX._SUM ;
                    
    LABEL 
    NUM     ='ACTUAL INCIDENT COUNT -NUMERATOR '
    DENOM   ='DENOMINATOR'
    USERS   ='PATIENT COUNT '
    OR      ='OBSERVED RATE ' 
    ER      ='EXPECTED RATE ' 
    RAR     ='RISK ADJUSTED RATE '
    SD_RAR  ='Standard deviation of RAR'
    LRAR    ='Lower bound of RAR'
    URAR    ='Upper bound of RAR'
; 
RUN;
%MEND STATELEVEL;


%STATELEVEL(PREFIX=scoreD_both,COMP_NUM=91,MSR=ACUTE,SUFFIX=0,SUFFIX2=MED_ONLY)
%STATELEVEL(PREFIX=scoreD_both,COMP_NUM=91,MSR=ACUTE,SUFFIX=1,SUFFIX2=MME)

%STATELEVEL(PREFIX=scoreD_both,COMP_NUM=92,MSR=CHRONIC,SUFFIX=0,SUFFIX2=MED_ONLY)
%STATELEVEL(PREFIX=scoreD_both,COMP_NUM=92,MSR=CHRONIC,SUFFIX=1,SUFFIX2=MME)

%STATELEVEL(PREFIX=out,COMP_NUM=93,MSR=PRU,SUFFIX=0,SUFFIX2=MED_ONLY)
%STATELEVEL(PREFIX=out,COMP_NUM=93,MSR=PRU,SUFFIX=1,SUFFIX2=MME)
