
%LET TAPQ = TAPQ01 TAPQ02 TAPQ03 TAPQ05 TAPQ07 TAPQ08 TAPQ10 TAPQ11 TAPQ12 TAPQ13 TAPQ14 TAPQ15 TAPQ16 TAPQ90 TAPQ91 TAPQ92 TAPQ93;
%LET CNT  = 17;

PROC FORMAT;
   VALUE MISFMT .='.'
                OTHER='Nonmissing';
   VALUE GT0FMT .='.'
                0='0'
                0<-HIGH='>0';
RUN;
          
TITLE3 "Identify/Combine the overlapping stays";
%MACRO OVERLAP;
PROC SORT DATA=&OUTFILE1.(KEEP=STATE MSIS_ID KEY ADMIT DISCH &TAPQ.) OUT=SORTED0;
   BY STATE MSIS_ID DESCENDING ADMIT DESCENDING DISCH DESCENDING KEY;
RUN;
DATA SORTED0;
   SET SORTED0;
   BY STATE MSIS_ID;
   LAG_ADMIT=LAG1(ADMIT);
   IF FIRST.MSIS_ID THEN LAG_ADMIT=.;
   /*IMPORTANT: can't use LAG function inside if statement, it will mess up the order!!!
   IF FIRST.MSIS_ID THEN LAG_ADMIT=.;
   ELSE LAG_ADMIT=LAG1(ADMIT);
   */
RUN;

PROC SORT DATA=SORTED0 OUT=SORTED(RENAME=(ADMIT=_ADMIT DISCH=_DISCH %DO I=1 %TO &CNT.; %SCAN(&TAPQ.,&I.)=_%SCAN(&TAPQ.,&I.) %END;)); 
   BY STATE MSIS_ID ADMIT DISCH KEY;
RUN;

DATA CLAIM_LVL PATIENT_LVL;
   SET SORTED;
   BY STATE MSIS_ID;

   RETAIN ADMIT_MIN DISCH_MAX %DO I=1 %TO &CNT.; %SCAN(&TAPQ.,&I.)_MAX %END;;

   LENGTH OVERLAP $8;

   /*If the patient has only one claim*/
   IF FIRST.MSIS_ID AND LAST.MSIS_ID THEN DO;
      CLMCNT_Patient=1;
      ADMIT = _ADMIT;
      DISCH = _DISCH;
      ADMIT_MIN = _ADMIT;
      DISCH_MAX = _DISCH;
      %DO I=1 %TO &CNT.; %SCAN(&TAPQ.,&I.)=_%SCAN(&TAPQ.,&I.); %END;
      CLMCNT_Episode=1;
      EPISODE=1;
      OVERLAP="ONECLAIM";
   END;
   /*If the patient has more than one claim*/
   ELSE DO;
      /*Initialize the admission, discharge date for the first episode */
      IF FIRST.MSIS_ID THEN DO;
         CLMCNT_Patient=1;
         ADMIT_MIN  = _ADMIT;
         DISCH_MAX  = _DISCH;
         %DO I=1 %TO &CNT.; %SCAN(&TAPQ.,&I.)_MAX=_%SCAN(&TAPQ.,&I.); %END;
         CLMCNT_Episode=1;
         EPISODE=1;
      END;
      ELSE DO;
         CLMCNT_Patient+1;

         /*If the following claim is overlapping with the previous claim, recalculate the disch_max, and increment CLMCNT by 1 */
         IF _ADMIT <= DISCH_MAX  THEN DO;
            DISCH_MAX = MAX(DISCH_MAX,_DISCH);
            %DO I = 1 %TO &CNT.;
               IF _%SCAN(&TAPQ.,&I.)=1 THEN %SCAN(&TAPQ.,&I.)_MAX=1;
            %END;
            CLMCNT_Episode+1;
         END;
         /*If the following claim is NOT overlapping with the previous claim, initialize a new episode, 
           therefore set CLMCNT_Episode=1 and increment EPISODE by 1 */
         ELSE IF _ADMIT > DISCH_MAX THEN DO;
            ADMIT_MIN = _ADMIT;
            DISCH_MAX = _DISCH;
            %DO I = 1 %TO &CNT.; %SCAN(&TAPQ.,&I.)_MAX= _%SCAN(&TAPQ.,&I.); %END;
            CLMCNT_Episode=1;
            EPISODE+1;
         END;
      END;

      /*For the patient with the mutiple claims, 
        if the next claim's admission date is after the maximum discharge of the current episode, or 
        if this is the last claim of the patient, 
        then retain the earliest admit date, the latest discharge date, and the largest TAPQxx flag 
        from the claims within an episode for the to-be-output claim 
        then output this claim by setting OVERLAP='KEEP'*/
      IF LAG_ADMIT > DISCH_MAX OR LAST.MSIS_ID THEN DO;
         ADMIT=ADMIT_MIN;
         DISCH=DISCH_MAX;
         %DO I = 1 %TO &CNT.; %SCAN(&TAPQ.,&I.)= %SCAN(&TAPQ.,&I.)_MAX; %END;
         IF CLMCNT_Episode>1 THEN COMBINE=1;
         OVERLAP='KEEP';
      END;
      ELSE OVERLAP='D'; /*To be dropped overlapping claims*/
   END;

   FORMAT ADMIT ADMIT_MIN _ADMIT DISCH DISCH_MAX _DISCH LAG_ADMIT MMDDYY10.;

   OUTPUT CLAIM_LVL;
   IF LAST.MSIS_ID THEN OUTPUT PATIENT_LVL;
RUN;

/*Attach the CLMCNT_Patient_TOT, EPISODE_TOT to the claim-level data*/
DATA MERGED1;
   MERGE CLAIM_LVL(IN=A) 
         PATIENT_LVL(IN=B KEEP=STATE MSIS_ID EPISODE CLMCNT_Patient 
                          RENAME=(EPISODE=EPISODE_TOT CLMCNT_Patient=CLMCNT_Patient_TOT));
   BY STATE MSIS_ID;
   IF A AND B;
RUN;

/*Attach the CLMCNT_Episode_TOT to the claim-level data*/
PROC SORT DATA=MERGED1;
   BY STATE MSIS_ID EPISODE;
RUN;
PROC SORT DATA=MERGED1(KEEP=STATE MSIS_ID OVERLAP EPISODE CLMCNT_Episode RENAME=CLMCNT_Episode=CLMCNT_Episode_TOT) 
   OUT=EPISODE_PATIENT(DROP=OVERLAP);
   WHERE OVERLAP IN ('KEEP','ONECLAIM');
   BY STATE MSIS_ID EPISODE;
RUN;
DATA MERGED2;
   MERGE MERGED1(IN=A) EPISODE_PATIENT(IN=B);
   BY STATE MSIS_ID EPISODE;
   IF A AND B;
RUN;

/*Attach other variables back to the claim-file
Drop the ADMIT DISCH &TAPQ. from the original file, keep the ones from the merged2 file*/
PROC SORT DATA=MERGED2 NODUPKEY;
   BY STATE MSIS_ID KEY;
RUN;
PROC SORT DATA=&OUTFILE1. NODUPKEY;
   BY STATE MSIS_ID KEY;
RUN;
DATA /*OUT1.*/&OUTFILE1._ALL;
   MERGE MERGED2(IN=A) &OUTFILE1.(IN=B DROP=ADMIT DISCH &TAPQ.);
   BY STATE MSIS_ID KEY;
   IF A AND B;
   IF ADMIT^=. THEN CHK_DIFF_ADMIT=_ADMIT-ADMIT;
   IF DISCH^=. THEN CHK_DIFF_DISCH=DISCH-_DISCH;
RUN;

/*Sort the data in this order for checking convinience later
Sort by _ADMIT, _DISCH instead of ADMIT, DISCH, since _ADMIT,_DISCH are the original.
ADMIT, DISCH are only nonmissing for the kept claims
*/
PROC SORT DATA=/*OUT1.*/&OUTFILE1._ALL;
   BY STATE MSIS_ID _ADMIT _DISCH KEY;
RUN;

/*Output the data files*/
DATA /*OUT1.*/&OUTFILE1.(KEEP=MSIS_ID STATE TAPQ01-TAPQ03 TAPQ05 TAPQ07 TAPQ08 TAPQ10-TAPQ16 TAPQ90-TAPQ93
                       KEY DRG DRGVER MDC SEX AGE AGEDAY PSTCO
                       RACE YEAR DQTR PAY1 PAY2
                       ASOURCE POINTOFORIGINUB04 ATYPE
                       DX1-DX&NDX. DXPOA1-DXPOA&NDX. PR1-PR&NPR. DISCWT
                       HICNO ADMIT DISCH HOSPID
                       A_DUAL FROM
                       ICDVER
                       EPISODE EPISODE_TOT CLMCNT_Episode_TOT CLMCNT_Patient_TOT OVERLAP COMBINE)
     /*OUT1.*/&OUTFILE1._DroppedOverlap;
   SET /*OUT1.*/&OUTFILE1._ALL;

   LABEL CLMCNT_Episode_TOT = 'Total # of claims in an episode'
         CLMCNT_Patient_TOT = 'Total # of claims for a patient'
         EPISODE_TOT        = 'Total # of episodes for a patient'
         COMBINE            = 'Indicates if the claim is a combined claim'
         OVERLAP            = 'Overlapping indicator: ONECLAIM, KEEP, D';

   IF OVERLAP IN ('KEEP','ONECLAIM') THEN OUTPUT /*OUT1.*/&OUTFILE1.;
   ELSE OUTPUT /*OUT1.*/&OUTFILE1._DroppedOverlap;
RUN;

/*Checking the following: If any of them doesn't hold, something is wrong!!!
1)ADMIT, DISCH are only nonmissing for the kept claims
2)For the claims with COMBINE=1,  CLMCNT_Episode_TOT and CLMCNT_Patient_TOT should be >1
3)For the claim with OVERLAP='D', CLMCNT_Episode_TOT and CLMCNT_Patient_TOT should be >1
4)For the claim with OVERLAP='KEEP', CLMCNT_Patient_TOT should be >1
5)For the claim with OVERLAP='ONECLAIM', CLMCNT_Patient_TOT and EPISODE_TOT should be 1
6)CLMCNT_Patient_TOT >= CLMCNT_Episode_TOT
7)CHK_DIFF_ADMIT, CHK_DIFF_DISCH shouldn't be <0*/

TITLE4 "Checking the overlapping variables";
PROC FREQ DATA=/*OUT1.*/&OUTFILE1._ALL;
   TABLES OVERLAP*ADMIT*DISCH*_ADMIT*_DISCH
          OVERLAP*COMBINE*ADMIT*DISCH*_ADMIT*_DISCH*CHK_DIFF_ADMIT*CHK_DIFF_DISCH
          COMBINE 
          OVERLAP*COMBINE OVERLAP*COMBINE*FROM
          COMBINE*CLMCNT_Episode_TOT 
          OVERLAP*EPISODE_TOT
          OVERLAP*COMBINE*CLMCNT_Patient_TOT*CLMCNT_Episode_TOT 
          OVERLAP*COMBINE*EPISODE_TOT*CLMCNT_Episode_TOT A_DUAL*OVERLAP*TAPQ93/MISSING LIST;
   FORMAT ADMIT DISCH _ADMIT _DISCH MISFMT. CHK_DIFF_ADMIT CHK_DIFF_DISCH GT0FMT.;
RUN;

TITLE4 "Spot-Check the patient='740150647014'";
PROC PRINT DATA=/*OUT1.*/&OUTFILE1._ALL;
VAR STATE MSIS_ID KEY  ADMIT DISCH _ADMIT _DISCH /*ADMIT_MIN DISCH_MAX*/ LAG_ADMIT TAPQ93 _TAPQ93 TAPQ93_MAX FROM
    COMBINE OVERLAP EPISODE CLMCNT_Episode CLMCNT_Patient EPISODE_TOT CLMCNT_Episode_TOT CLMCNT_Patient_TOT;
WHERE MSIS_ID='740150647014';
RUN;

TITLE4 "Spot-Check the patient with CLMCNT_Patient_TOT>=20";
PROC PRINT DATA=/*OUT1.*/&OUTFILE1._ALL(OBS=200);
VAR STATE MSIS_ID KEY  ADMIT DISCH _ADMIT _DISCH /*ADMIT_MIN DISCH_MAX*/ LAG_ADMIT TAPQ93 _TAPQ93 TAPQ93_MAX FROM
    COMBINE OVERLAP EPISODE CLMCNT_Episode CLMCNT_Patient EPISODE_TOT CLMCNT_Episode_TOT CLMCNT_Patient_TOT;
WHERE CLMCNT_Patient_TOT>=20;
RUN;

TITLE4 "Spot-Check the patient in VA with CLMCNT_Patient_TOT>=20";
PROC PRINT DATA=/*OUT1.*/&OUTFILE1._ALL(OBS=200);
VAR STATE MSIS_ID KEY  ADMIT DISCH _ADMIT _DISCH /*ADMIT_MIN DISCH_MAX*/ LAG_ADMIT TAPQ93 _TAPQ93 TAPQ93_MAX FROM
    COMBINE OVERLAP EPISODE CLMCNT_Episode CLMCNT_Patient EPISODE_TOT CLMCNT_Episode_TOT CLMCNT_Patient_TOT;
WHERE STATE='VA' AND CLMCNT_Patient_TOT>=20;
RUN;

TITLE4 "Spot-Check the patient in VA with CLMCNT_Patient_TOT>=40";
PROC PRINT DATA=/*OUT1.*/&OUTFILE1._ALL(OBS=400);
VAR STATE MSIS_ID KEY  ADMIT DISCH _ADMIT _DISCH /*ADMIT_MIN DISCH_MAX*/ LAG_ADMIT TAPQ93 _TAPQ93 TAPQ93_MAX FROM
    COMBINE OVERLAP EPISODE CLMCNT_Episode CLMCNT_Patient EPISODE_TOT CLMCNT_Episode_TOT CLMCNT_Patient_TOT;
WHERE STATE='VA' AND CLMCNT_Patient_TOT>=40;
RUN;

TITLE4 "Checking the counts: SUM should be # of claims in &OUTFILE1._ALL";
PROC MEANS DATA=/*OUT1.*/&OUTFILE1._ALL N SUM;
   WHERE OVERLAP IN ('KEEP','ONECLAIM');
   VAR CLMCNT_Episode_TOT;
RUN;

PROC SORT DATA=/*OUT1.*/&OUTFILE1._ALL OUT=TEMP NODUPKEY;
   BY STATE MSIS_ID ;
RUN;
PROC MEANS DATA=TEMP N SUM;
   VAR CLMCNT_Patient_TOT;
RUN;

TITLE3 "Check TAPQ93 calculation in the combined stays";
PROC SORT DATA=/*OUT1.*/&OUTFILE1._ALL OUT=SORTEDCHK1;
   BY STATE MSIS_ID EPISODE;
RUN;
PROC MEANS DATA=SORTEDCHK1 NOPRINT;
   BY STATE MSIS_ID EPISODE; 
   VAR _TAPQ93; 
   OUTPUT OUT=CHKTAPQ93 SUM=SUM;
RUN;
DATA CHKTAPQ93;
   SET CHKTAPQ93;
   IF SUM>=1 THEN CHKTAPQ93=1;
RUN;
PROC SORT DATA=/*OUT1.*/&OUTFILE1. OUT=SORTEDCHK2 NODUPKEY; /*SHOULD HAVE NO DUPS*/
   BY STATE MSIS_ID EPISODE;
RUN;
DATA MERGEDCHK;
   MERGE SORTEDCHK2(IN=A) CHKTAPQ93(IN=B);
   BY STATE MSIS_ID EPISODE;
   IF A AND B;
   DIFF_TAPQ93=CHKTAPQ93-TAPQ93;
RUN;

TITLE3 "sum of product of sum and frequency count should be same as the # of PU in the first table of program 3";
PROC FREQ DATA=MERGEDCHK;
   TABLES DIFF_TAPQ93*CHKTAPQ93*TAPQ93 DIFF_TAPQ93*SUM*CHKTAPQ93*TAPQ93/MISSING LIST;
RUN;

TITLE3 "Check _TAPQ93 for the dropped overlappign stays with A_DUAL=0 in /*OUT1.*/&OUTFILE1._ALL";
PROC FREQ DATA=/*OUT1.*/&OUTFILE1._ALL;
   TABLES A_DUAL*FROM*OVERLAP*COMBINE*_TAPQ93
          STATE*A_DUAL*FROM*OVERLAP*COMBINE*_TAPQ93/MISSING LIST;
RUN;
PROC FREQ DATA=/*OUT1.*/&OUTFILE1._ALL;
   TABLES STATE*A_DUAL*FROM*OVERLAP*COMBINE*_TAPQ93/MISSING LIST;
   WHERE _TAPQ93=1 AND A_DUAL=0 AND OVERLAP='D';
RUN;

/*End of checking*/

%MEND OVERLAP;
%OVERLAP;




